/*-
 * Copyright (c) 1999 James Howard and Dag-Erling Codan Smrgrav
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>

#ifndef lint
#if 0
__RCSID("$OpenBSD: binary.c,v 1.15 2005/04/25 08:21:20 otto Exp $");
#else
__FBSDID("$FreeBSD$");
#endif
#endif /* not lint */

#include <ctype.h>
#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <wchar.h>
#include <wctype.h>

#include "grep.h"

#define	iswbinary(ch)	(!iswspace((ch)) && iswcntrl((ch)))

int
bin_file(FILE *f)
{
	wint_t	 ch = L'\0';
	size_t	 i;
	int	 ret = 0;

	if (fseek(f, 0L, SEEK_SET) == -1)
		return (0);

	for (i = 0; (i <= BUFSIZ) && (ch != WEOF); i++) {
		ch = fgetwc(f);
		if (iswbinary(ch)) {
			ret = 1;
			break;
		}
	}

	rewind(f);
	return (ret);
}

int
mmbin_file(struct mmfile *f)
{
	int	 i;
	wchar_t	*wbuf;
	size_t	 s;
	
	if ((s = mbstowcs(NULL, f->base, 0)) == -1)
		return (0);

	wbuf = grep_malloc((s + 1) * sizeof(wchar_t));

	if (mbstowcs(wbuf, f->base, s) == -1)
		return (0);

	/* XXX knows too much about mmf internals */
	for (i = 0; i < BUFSIZ && i < f->len; i++)
		if (iswbinary(wbuf[i])) {
			free(wbuf);
			return (1);
	}
	free(wbuf);
	return (0);
}
